@@ -18,21 +18,34 @@ from qtpy.compat import (getexistingdirectory, getopenfilename, from_qvariant,
                          to_qvariant)
 from qtpy.QtCore import Qt, QRegularExpression, QSize, Signal, Slot
 from qtpy.QtGui import QColor, QRegularExpressionValidator, QTextOption
-from qtpy.QtWidgets import (QAction, QButtonGroup, QCheckBox, QDoubleSpinBox,
-                            QFileDialog, QGridLayout, QGroupBox,
-                            QHBoxLayout, QLabel, QLineEdit, QMessageBox,
-                            QPlainTextEdit, QPushButton, QRadioButton,
-                            QSpinBox, QTabWidget, QVBoxLayout, QWidget)
+from qtpy.QtWidgets import (
+    QAction,
+    QButtonGroup,
+    QCheckBox,
+    QDoubleSpinBox,
+    QFileDialog,
+    QGroupBox,
+    QHBoxLayout,
+    QLabel,
+    QLayout,
+    QLineEdit,
+    QMessageBox,
+    QPlainTextEdit,
+    QPushButton,
+    QRadioButton,
+    QSpinBox,
+    QTabWidget,
+    QVBoxLayout,
+    QWidget,
+)
 
 # Local imports
 from spyder.api.widgets.comboboxes import SpyderComboBox, SpyderFontComboBox
-from spyder.config.base import _
+from spyder.config.base import _, get_home_dir
 from spyder.config.manager import CONF
 from spyder.config.user import NoDefault
-from spyder.py3compat import to_text_string
 from spyder.utils.icon_manager import ima
-from spyder.utils.misc import getcwd_or_home
-from spyder.utils.stylesheet import AppStyle
+from spyder.utils.stylesheet import AppStyle, MAC, WIN
 from spyder.widgets.colors import ColorLayout
 from spyder.widgets.helperwidgets import TipWidget, ValidationLineEdit
 from spyder.widgets.comboboxes import FileComboBox
@@ -162,11 +175,20 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
             if self.CONF_SECTION == 'main':
                 self._save_lang()
 
+            restart = False
             for restart_option in self.restart_options:
                 if restart_option in self.changed_options:
-                    self.prompt_restart_required()
+                    restart = self.prompt_restart_required()
                     break  # Ensure a single popup is displayed
-            self.set_modified(False)
+
+            # Don't call set_modified() when restart() is called: The
+            # latter triggers closing of the application. Calling the former
+            # afterwards may result in an error because the underlying C++ Qt
+            # object of 'self' may be deleted at that point.
+            if restart:
+                self.restart()
+            else:
+                self.set_modified(False)
 
     def check_settings(self):
         """This method is called to check settings after configuration
@@ -185,7 +207,7 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
         for lineedit in self.lineedits:
             if lineedit in self.validate_data and lineedit.isEnabled():
                 validator, invalid_msg = self.validate_data[lineedit]
-                text = to_text_string(lineedit.text())
+                text = str(lineedit.text())
                 if not validator(text):
                     QMessageBox.critical(self, self.get_name(),
                                          f"{invalid_msg}:<br><b>{text}</b>",
@@ -273,7 +295,7 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
             if getattr(textedit, 'content_type', None) == list:
                 data = ', '.join(data)
             elif getattr(textedit, 'content_type', None) == dict:
-                data = to_text_string(data)
+                data = str(data)
             textedit.setPlainText(data)
             textedit.textChanged.connect(lambda opt=option, sect=sec:
                                          self.has_been_modified(sect, opt))
@@ -291,11 +313,11 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
         for combobox, (sec, option, default) in list(self.comboboxes.items()):
             value = self.get_option(option, default, section=sec)
             for index in range(combobox.count()):
-                data = from_qvariant(combobox.itemData(index), to_text_string)
+                data = from_qvariant(combobox.itemData(index), str)
                 # For PyQt API v2, it is necessary to convert `data` to
                 # unicode in case the original type was not a string, like an
                 # integer for example (see qtpy.compat.from_qvariant):
-                if to_text_string(data) == to_text_string(value):
+                if str(data) == str(value):
                     break
             else:
                 if combobox.count() == 0:
@@ -380,7 +402,7 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
                 option in self.changed_options
                 or (sec, option) in self.changed_options
                 or not self.LOAD_FROM_CONFIG
-            ):
+            ) and option is not None:
                 self.set_option(option, radiobutton.isChecked(), section=sec,
                                 recursive_notification=False)
 
@@ -395,7 +417,7 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
                 if content_type == list:
                     data = [item.strip() for item in data.split(',')]
                 else:
-                    data = to_text_string(data)
+                    data = str(data)
 
                 self.set_option(
                     option,
@@ -423,7 +445,7 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
                 elif content_type in (tuple, list):
                     data = [item.strip() for item in data.split(',')]
                 else:
-                    data = to_text_string(data)
+                    data = str(data)
                 self.set_option(option, data, section=sec,
                                 recursive_notification=False)
 
@@ -443,7 +465,7 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
                 or not self.LOAD_FROM_CONFIG
             ):
                 data = combobox.itemData(combobox.currentIndex())
-                self.set_option(option, from_qvariant(data, to_text_string),
+                self.set_option(option, from_qvariant(data, str),
                                 section=sec, recursive_notification=False)
 
         for (fontbox, sizebox), option in list(self.fontboxes.items()):
@@ -459,7 +481,7 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
                 or not self.LOAD_FROM_CONFIG
             ):
                 self.set_option(option,
-                                to_text_string(clayout.lineedit.text()),
+                                str(clayout.lineedit.text()),
                                 section=sec, recursive_notification=False)
 
         for (clayout, cb_bold, cb_italic), (sec, option, _default) in list(
@@ -469,7 +491,7 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
                 or (sec, option) in self.changed_options
                 or not self.LOAD_FROM_CONFIG
             ):
-                color = to_text_string(clayout.lineedit.text())
+                color = str(clayout.lineedit.text())
                 bold = cb_bold.isChecked()
                 italic = cb_italic.isChecked()
                 self.set_option(option, (color, bold, italic), section=sec,
@@ -529,7 +551,7 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
     def create_radiobutton(self, text, option, default=NoDefault,
                            tip=None, msg_warning=None, msg_info=None,
                            msg_if_enabled=False, button_group=None,
-                           restart=False, section=None):
+                           restart=False, section=None, id_=None):
         layout = QHBoxLayout()
         layout.setContentsMargins(0, 0, 0, 0)
         radiobutton = QRadioButton(text)
@@ -537,12 +559,19 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
 
         if section is not None and section != self.CONF_SECTION:
             self.cross_section_options[option] = section
+
         if button_group is None:
             if self.default_button_group is None:
                 self.default_button_group = QButtonGroup(self)
             button_group = self.default_button_group
-        button_group.addButton(radiobutton)
+
+        if id_ is None:
+            button_group.addButton(radiobutton)
+        else:
+            button_group.addButton(radiobutton, id=id_)
+
         self.radiobuttons[radiobutton] = (section, option, default)
+
         if msg_warning is not None or msg_info is not None:
             def show_message(is_checked):
                 if is_checked or not msg_if_enabled:
@@ -553,15 +582,18 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
                         QMessageBox.information(self, self.get_name(),
                                                 msg_info, QMessageBox.Ok)
             radiobutton.toggled.connect(show_message)
+
         radiobutton.restart_required = restart
         radiobutton.label_text = text
 
         if tip is not None:
             layout, help_label = self.add_help_info_label(layout, tip)
             radiobutton.help_label = help_label
+
         widget = QWidget(self)
         widget.radiobutton = radiobutton
         widget.setLayout(layout)
+
         return widget
 
     def create_lineedit(self, text, option, default=NoDefault,
@@ -709,14 +741,15 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
         )
 
         if alignment == Qt.Vertical:
-            # This is necessary to position browse_btn vertically centered with
-            # respect to the lineedit.
-            browse_btn.setStyleSheet("margin-top: 28px")
+            button_layout = QVBoxLayout()
+            button_layout.setContentsMargins(0, 0, 0, 0)
+            button_layout.addWidget(QLabel(""))
+            button_layout.addWidget(browse_btn)
 
-            layout = QGridLayout()
+            layout = QHBoxLayout()
             layout.setContentsMargins(0, 0, 0, 0)
-            layout.addWidget(widget, 0, 0)
-            layout.addWidget(browse_btn, 0, 1)
+            layout.addWidget(widget)
+            layout.addLayout(button_layout)
         else:
             # This is necessary to position browse_btn vertically centered with
             # respect to the lineedit.
@@ -739,9 +772,9 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
 
     def select_directory(self, edit):
         """Select directory"""
-        basedir = to_text_string(edit.text())
+        basedir = str(edit.text())
         if not osp.isdir(basedir):
-            basedir = getcwd_or_home()
+            basedir = get_home_dir()
         title = _("Select directory")
         directory = getexistingdirectory(self, title, basedir)
         if directory:
@@ -777,14 +810,15 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
         )
 
         if alignment == Qt.Vertical:
-            # This is necessary to position browse_btn vertically centered with
-            # respect to the lineedit.
-            browse_btn.setStyleSheet("margin-top: 28px")
+            button_layout = QVBoxLayout()
+            button_layout.setContentsMargins(0, 0, 0, 0)
+            button_layout.addWidget(QLabel(""))
+            button_layout.addWidget(browse_btn)
 
-            layout = QGridLayout()
+            layout = QHBoxLayout()
             layout.setContentsMargins(0, 0, 0, 0)
-            layout.addWidget(widget, 0, 0)
-            layout.addWidget(browse_btn, 0, 1)
+            layout.addWidget(widget)
+            layout.addLayout(button_layout)
         else:
             # This is necessary to position browse_btn vertically centered with
             # respect to the lineedit.
@@ -807,9 +841,9 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
 
     def select_file(self, edit, filters=None, **kwargs):
         """Select File"""
-        basedir = osp.dirname(to_text_string(edit.text()))
+        basedir = osp.dirname(str(edit.text()))
         if not osp.isdir(basedir):
-            basedir = getcwd_or_home()
+            basedir = get_home_dir()
         if filters is None:
             filters = _("All files (*)")
         title = _("Select file")
@@ -916,15 +950,18 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
 
     def create_combobox(self, text, choices, option, default=NoDefault,
                         tip=None, restart=False, section=None,
-                        items_elide_mode=None):
+                        items_elide_mode=None, alignment=Qt.Horizontal):
         """choices: couples (name, key)"""
         if section is not None and section != self.CONF_SECTION:
             self.cross_section_options[option] = section
+
+        # Widgets
         label = QLabel(text)
         combobox = SpyderComboBox(items_elide_mode=items_elide_mode)
         for name, key in choices:
             if not (name is None and key is None):
                 combobox.addItem(name, to_qvariant(key))
+
         # Insert separators
         count = 0
         for index, item in enumerate(choices):
@@ -933,20 +970,42 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
                 combobox.insertSeparator(index + count)
                 count += 1
         self.comboboxes[combobox] = (section, option, default)
-        layout = QHBoxLayout()
-        layout.addWidget(label)
-        layout.addWidget(combobox)
-        layout.addStretch(1)
+
+        if alignment == Qt.Vertical:
+            layout = QVBoxLayout()
+
+            if tip is not None:
+                label_layout = QHBoxLayout()
+                label_layout.setSpacing(0)
+                label_layout.addWidget(label)
+                label_layout, help_label = self.add_help_info_label(
+                    label_layout, tip
+                )
+                layout.addLayout(label_layout)
+            else:
+                layout.addWidget(label)
+
+            layout.addWidget(combobox)
+        else:
+            layout = QHBoxLayout()
+            layout.addWidget(label)
+            layout.addWidget(combobox)
+            if tip is not None:
+                layout, help_label = self.add_help_info_label(layout, tip)
+            layout.addStretch(1)
+
         layout.setContentsMargins(0, 0, 0, 0)
+
         widget = QWidget(self)
         widget.label = label
         widget.combobox = combobox
         if tip is not None:
-            layout, help_label = self.add_help_info_label(layout, tip)
             widget.help_label = help_label
+
         widget.setLayout(layout)
         combobox.restart_required = restart
         combobox.label_text = text
+
         return widget
 
     def create_file_combobox(self, text, choices, option, default=NoDefault,
@@ -1109,15 +1168,31 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
         tab = QWidget(self)
         layout = QVBoxLayout()
         layout.setContentsMargins(0, 0, 0, 0)
+
+        # This is necessary to make Qt respect the declared vertical spacing
+        # for widgets. In other words, it prevents text to be cropped when the
+        # total height of the page is too large.
+        layout.setSizeConstraint(QLayout.SetFixedSize)
+
         for w in widgets:
+            # We need to set a min width so that pages are not shown too thin
+            # due to setting the layout size constraint above.
+            w.setMinimumWidth(
+                self.MAX_WIDTH - (60 if MAC else (80 if WIN else 70))
+            )
             layout.addWidget(w)
+
         layout.addStretch(1)
         tab.setLayout(layout)
 
         self.tabs.addTab(tab, name)
 
-    def prompt_restart_required(self):
-        """Prompt the user with a request to restart."""
+    def prompt_restart_required(self) -> bool:
+        """
+        Prompt the user with a request to restart.
+        
+        It returns ``True`` when the request is accepted, ``False`` otherwise.
+        """
         message = _(
             "One or more of the settings you changed requires a restart to be "
             "applied.<br><br>"
@@ -1131,8 +1206,7 @@ class SpyderConfigPage(SidebarPage, ConfigAccessMixin):
             QMessageBox.Yes | QMessageBox.No
         )
 
-        if answer == QMessageBox.Yes:
-            self.restart()
+        return answer == QMessageBox.Yes
 
     def restart(self):
         """Restart Spyder."""
